# -*- coding: utf-8 -*-

import datetime
import constants
from grandfatherson import to_delete, to_keep


def filter_delete_filename_list(filename_list, keeps):
    """
    获取需要删除的
    """
    time_to_filename = dict()

    for filename in filename_list:
        slist = filename.split('.')
        if len(slist) < 2:
            #like yb.20140516_114126.tar
            continue

        str_datetime = slist[-2]

        dt = datetime.datetime.strptime(str_datetime, constants.STRFTIME_TPL)

        time_to_filename[dt] = filename

    # 要删除的时间，如果同一天有好多个的话，只保留第一个
    delete_times = to_delete(time_to_filename.keys(), **keeps)

    return [time_to_filename[it] for it in delete_times]



# -*- coding: utf-8 -*-

# 转化成time的格式
STRFTIME_TPL = '%Y%m%d_%H%M%S'

# 需要保留的时间
# 看源码可知，是用如下代码实现的:
#    return (filters.Years.filter(datetimes, number=years, now=now) |
#            filters.Months.filter(datetimes, number=months, now=now) |
#            filters.Weeks.filter(datetimes, number=weeks,
#                                 firstweekday=firstweekday, now=now) |
#            filters.Days.filter(datetimes, number=days, now=now) |
#            filters.Hours.filter(datetimes, number=hours, now=now) |
#            filters.Minutes.filter(datetimes, number=minutes, now=now) |
#            filters.Seconds.filter(datetimes, number=seconds, now=now))
# 比如 days=1，由于days的mark会替换hours之类的都为0，所以就代表保留最近一天的第一个
# 又比如 days=1, hours=2，则代表保留最近一天的第一个，以及最近两小时的第一个
# years 和 months 会不太一样，具体参考其每个的mark函数
# 所以下面的写法，相当于保存: 最近3个月的每月第一个，最近4周的每周第一个，最近7天的每天第一个，最近3600*24秒的的每秒第一个
# 即一天内多个的话，会被删除为只剩下第一个

# 已验证，确实是第一个而不是最后一个
KEEPS_NORMAL = dict(
    months=3,
    weeks=4,
    days=7,
    seconds=3600*24,
)



# -*- coding: utf-8 -*-

import sys
import os
import errno
import paramiko
from .base import BaseBackend
from ..utils import filter_delete_filename_list


class SFTPBackend(BaseBackend):
    """
    ssh后端
    """

    host = None
    port = None
    username = None
    password = None
    remote_dir = None
    transport = None
    sftp = None

    def __init__(self, host, username, password, remote_dir, port=None):
        super(SFTPBackend, self).__init__()
        self.host = host
        self.port = port or 22
        self.username = username
        self.password = password
        self.remote_dir = remote_dir

        # 这一步会建立连接
        try:
            self.transport = paramiko.Transport((self.host, self.port))
            # 登录验证
            self.transport.connect(username=self.username, password=self.password)
            self.sftp = paramiko.SFTPClient.from_transport(self.transport)
        except:
            # 这个的关闭，并不会关闭transport的连接
            if self.sftp:
                self.sftp.close()

            # 所以如果登录失败，要记得关闭连接
            if self.transport:
                self.transport.close()

            t, v, tb = sys.exc_info()
            raise t, v, tb

    def upload(self, file_path, category):
        """
        上传
        """
        filename = os.path.basename(file_path)

        dst_dir = os.path.join(self.remote_dir, category)
        try:
            self.sftp.stat(dst_dir)
        except IOError, e:
            # 说明没有文件
            if e.errno == errno.ENOENT:
                self.sftp.mkdir(dst_dir)
            else:
                raise e

        remote_path = os.path.join(dst_dir, filename)
        self.sftp.put(file_path, remote_path)

    def clean(self, category, keeps):
        dst_dir = os.path.join(self.remote_dir, category)

        delete_filename_list = filter_delete_filename_list(self.sftp.listdir(dst_dir), keeps)
        for filename in delete_filename_list:
            self.sftp.remove(os.path.join(dst_dir, filename))



# -*- coding: utf-8 -*-
"""
阿里云 OSS SDK: https://docs.aliyun.com
"""

import os
import oss2
from .base import BaseBackend
from ..utils import filter_delete_filename_list


class OSSBackend(BaseBackend):
    """
    阿里OSS后端，依赖 oss v2
    pip install oss2
    """

    bucket = None

    def __init__(self, access_key_id, access_key_secret, host, bucket_name):
        """
        access_key_id:
        access_key_secret:
        host: 域名，如 http://oss-cn-hangzhou.aliyuncs.com
        bucket_name: 不需要在后台自动创建，也会自动创建好
        """
        super(OSSBackend, self).__init__()
        auth = oss2.Auth(access_key_id, access_key_secret)
        self.bucket = oss2.Bucket(auth, host, bucket_name)
        # 无论是否已经存在，都自动创建一次，不会报错.
        self.bucket.create_bucket()

    def upload(self, file_path, category):
        """
        上传
        """

        filename = os.path.basename(file_path)

        rsp = self.bucket.put_object_from_file(
            os.path.join(category, filename),
            file_path,
        )

        if rsp.status != 200:
            raise Exception('put_object_from_file fail: <%s> %s' % (rsp.status, rsp.read()))

    def clean(self, category, keeps):
        object_list = [obj.key for obj in self.bucket.list_objects(category+'/').object_list]
        delete_filename_list = filter_delete_filename_list(object_list, keeps)
        if delete_filename_list:
            self.bucket.batch_delete_objects(delete_filename_list)
